/*************************************************************************
 * Copyright 2008 Regents of the University of California
 * Copyright 2009-2012 Ent. Services Development Corporation LP
 *
 * Redistribution and use of this software in source and binary forms,
 * with or without modification, are permitted provided that the
 * following conditions are met:
 *
 *   Redistributions of source code must retain the above copyright
 *   notice, this list of conditions and the following disclaimer.
 *
 *   Redistributions in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimer
 *   in the documentation and/or other materials provided with the
 *   distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 * ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE. USERS OF THIS SOFTWARE ACKNOWLEDGE
 * THE POSSIBLE PRESENCE OF OTHER OPEN SOURCE LICENSED MATERIAL,
 * COPYRIGHTED MATERIAL OR PATENTED MATERIAL IN THIS SOFTWARE,
 * AND IF ANY SUCH MATERIAL IS DISCOVERED THE PARTY DISCOVERING
 * IT MAY INFORM DR. RICH WOLSKI AT THE UNIVERSITY OF CALIFORNIA,
 * SANTA BARBARA WHO WILL THEN ASCERTAIN THE MOST APPROPRIATE REMEDY,
 * WHICH IN THE REGENTS' DISCRETION MAY INCLUDE, WITHOUT LIMITATION,
 * REPLACEMENT OF THE CODE SO IDENTIFIED, LICENSING OF THE CODE SO
 * IDENTIFIED, OR WITHDRAWAL OF THE CODE CAPABILITY TO THE EXTENT
 * NEEDED TO COMPLY WITH ANY SUCH LICENSES OR RIGHTS.
 ************************************************************************/

package com.eucalyptus.blockstorage.entities;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.Index;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.PersistenceContext;
import javax.persistence.Table;

import com.eucalyptus.entities.AbstractPersistent;

@Entity
@PersistenceContext(name = "eucalyptus_storage")
@Table(name = "volume_exports", indexes = {
    @Index( name = "volume_exports_volume_token_idx", columnList = "token" ),
    @Index( name = "volume_exports_volume_idx", columnList = "volume"  )
})
public class VolumeExportRecord extends AbstractPersistent {
  private static final long serialVersionUID = 1L;

  @ManyToOne
  @JoinColumn(name = "token", nullable = false)
  private VolumeToken token;

  @ManyToOne
  @JoinColumn(name = "volume", nullable = false)
  private VolumeInfo volume;

  @Column(name = "host_ip", updatable = false, nullable = false)
  private String hostIp;

  @Column(name = "host_iqn", updatable = false, nullable = false)
  private String hostIqn;

  @Column(name = "is_active")
  private Boolean isActive;

  @Column(name = "connection_string", length = 4096)
  private String connectionString;

  public VolumeExportRecord() {
    token = null;
    hostIp = null;
    hostIqn = null;
    volume = null;
  }

  public VolumeExportRecord(VolumeInfo vol, VolumeToken tok, String ip, String iqn) {
    this.volume = vol;
    this.token = tok;
    this.hostIp = ip;
    this.hostIqn = iqn;
  }

  public Boolean getIsActive() {
    return isActive;
  }

  public void setIsActive(Boolean isActive) {
    this.isActive = isActive;
  }

  public VolumeToken getToken() {
    return token;
  }

  public void setToken(VolumeToken token) {
    this.token = token;
  }

  public String getHostIp() {
    return hostIp;
  }

  public void setHostIp(String hostIp) {
    this.hostIp = hostIp;
  }

  public String getHostIqn() {
    return hostIqn;
  }

  public void setHostIqn(String hostIqn) {
    this.hostIqn = hostIqn;
  }

  @Override
  public int hashCode() {
    final int prime = 31;
    int result = 1;
    result = prime * result + ((this.getId() == null) ? 0 : this.getId().hashCode());
    return result;
  }

  @Override
  public boolean equals(Object obj) {
    if (this == obj) {
      return true;
    }
    if (obj == null) {
      return false;
    }
    if (getClass() != obj.getClass()) {
      return false;
    }
    VolumeExportRecord other = (VolumeExportRecord) obj;
    if (this.volume == null) {
      if (other.volume != null) {
        return false;
      }
    } else if (!this.volume.equals(other.volume) || !this.hostIqn.equals(other.hostIqn) || !this.hostIp.equals(other.hostIp)
        || !this.isActive == other.isActive) {
      return false;
    }
    return true;
  }

  public String getConnectionString() {
    return connectionString;
  }

  public void setConnectionString(String connectionString) {
    this.connectionString = connectionString;
  }

  public VolumeInfo getVolume() {
    return volume;
  }

  public void setVolume(VolumeInfo volume) {
    this.volume = volume;
  }

}
