/*************************************************************************
 * Copyright 2008 Regents of the University of California
 * Copyright 2009 Ent. Services Development Corporation LP
 *
 * Redistribution and use of this software in source and binary forms,
 * with or without modification, are permitted provided that the
 * following conditions are met:
 *
 *   Redistributions of source code must retain the above copyright
 *   notice, this list of conditions and the following disclaimer.
 *
 *   Redistributions in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimer
 *   in the documentation and/or other materials provided with the
 *   distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 * ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE. USERS OF THIS SOFTWARE ACKNOWLEDGE
 * THE POSSIBLE PRESENCE OF OTHER OPEN SOURCE LICENSED MATERIAL,
 * COPYRIGHTED MATERIAL OR PATENTED MATERIAL IN THIS SOFTWARE,
 * AND IF ANY SUCH MATERIAL IS DISCOVERED THE PARTY DISCOVERING
 * IT MAY INFORM DR. RICH WOLSKI AT THE UNIVERSITY OF CALIFORNIA,
 * SANTA BARBARA WHO WILL THEN ASCERTAIN THE MOST APPROPRIATE REMEDY,
 * WHICH IN THE REGENTS' DISCRETION MAY INCLUDE, WITHOUT LIMITATION,
 * REPLACEMENT OF THE CODE SO IDENTIFIED, LICENSING OF THE CODE SO
 * IDENTIFIED, OR WITHDRAWAL OF THE CODE CAPABILITY TO THE EXTENT
 * NEEDED TO COMPLY WITH ANY SUCH LICENSES OR RIGHTS.
 ************************************************************************/
/*
 *
 * Author: Neil Soman neil@eucalyptus.com
 */

package com.eucalyptus.blockstorage.san.common;

import java.util.List;

import org.apache.log4j.Logger;

import com.eucalyptus.blockstorage.san.common.entities.SANInfo;
import com.google.common.collect.Lists;

public class Paths {
  private static final Logger LOG = Logger.getLogger(Paths.class);

  protected String currentPathsProperty = null;
  protected List<SANInfo.Path> paths = Lists.newArrayList();

  public synchronized int size() {
    return paths.size();
  }

  public synchronized Paths update(String value) {
    if (value != null && !value.equals(this.currentPathsProperty)) {
      try {
        List<SANInfo.Path> newPaths = SANInfo.parsePaths(value);
        this.paths.clear();
        this.paths.addAll(newPaths);
        this.currentPathsProperty = value;
      } catch (Exception e) {
        LOG.error(e, e);
      }
    }
    return this;
  }

  public synchronized String getPathsForVolumeProperty() {
    StringBuilder sb = new StringBuilder();
    for (SANInfo.Path path : this.paths) {
      if (sb.length() > 0) {
        sb.append(',');
      }
      sb.append(path.iface).append(',').append(path.ip).append(',').append(path.iqn);
    }
    return sb.toString();
  }

  public synchronized String getPathsForVolumeProperty(String iqn) {
    StringBuilder sb = new StringBuilder();
    for (SANInfo.Path path : this.paths) {
      if (sb.length() > 0) {
        sb.append(',');
      }
      sb.append(path.iface).append(',').append(path.ip).append(',').append(iqn);
    }
    return sb.toString();
  }

  public synchronized String getFirstTargetIqn() {
    if (this.paths.size() > 0) {
      return this.paths.get(0).iqn;
    }
    return null;
  }
}
