/*************************************************************************
 * Copyright 2008 Regents of the University of California
 * Copyright 2009-2012 Ent. Services Development Corporation LP
 *
 * Redistribution and use of this software in source and binary forms,
 * with or without modification, are permitted provided that the
 * following conditions are met:
 *
 *   Redistributions of source code must retain the above copyright
 *   notice, this list of conditions and the following disclaimer.
 *
 *   Redistributions in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimer
 *   in the documentation and/or other materials provided with the
 *   distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 * ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE. USERS OF THIS SOFTWARE ACKNOWLEDGE
 * THE POSSIBLE PRESENCE OF OTHER OPEN SOURCE LICENSED MATERIAL,
 * COPYRIGHTED MATERIAL OR PATENTED MATERIAL IN THIS SOFTWARE,
 * AND IF ANY SUCH MATERIAL IS DISCOVERED THE PARTY DISCOVERING
 * IT MAY INFORM DR. RICH WOLSKI AT THE UNIVERSITY OF CALIFORNIA,
 * SANTA BARBARA WHO WILL THEN ASCERTAIN THE MOST APPROPRIATE REMEDY,
 * WHICH IN THE REGENTS' DISCRETION MAY INCLUDE, WITHOUT LIMITATION,
 * REPLACEMENT OF THE CODE SO IDENTIFIED, LICENSING OF THE CODE SO
 * IDENTIFIED, OR WITHDRAWAL OF THE CODE CAPABILITY TO THE EXTENT
 * NEEDED TO COMPLY WITH ANY SUCH LICENSES OR RIGHTS.
 ************************************************************************/

package com.eucalyptus.compute.common.internal.images;

import javax.persistence.Column;
import javax.persistence.DiscriminatorColumn;
import javax.persistence.DiscriminatorType;
import javax.persistence.DiscriminatorValue;
import javax.persistence.Entity;
import javax.persistence.EnumType;
import javax.persistence.Enumerated;
import javax.persistence.Inheritance;
import javax.persistence.InheritanceType;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.PersistenceContext;
import javax.persistence.Table;
import com.eucalyptus.compute.common.ImageMetadata;
import com.eucalyptus.compute.common.ImageMetadata.DeviceMappingType;
import com.eucalyptus.entities.AbstractPersistent;

/**
 * @see <a href="http://docs.aws.amazon.com/AWSEC2/latest/UserGuide/block-device-mapping-concepts.html">Block Device Mappings</a>
 */
@Entity
@PersistenceContext( name = "eucalyptus_cloud" )
@Table( name = "metadata_device_mappings" )
@Inheritance( strategy = InheritanceType.SINGLE_TABLE )
@DiscriminatorColumn( name = "metadata_device_mapping_discriminator", discriminatorType = DiscriminatorType.STRING )
@DiscriminatorValue( value = "basic" )
public class DeviceMapping extends AbstractPersistent {
  
  @Column( name = "metadata_device_mapping_type" )
  @Enumerated( EnumType.STRING )
  private ImageMetadata.DeviceMappingType deviceMappingType;
  
  @Column( name = "metadata_device_mapping_device_name" )
  private String                  deviceName;

  @Column( name = "metadata_device_mapping_virtual_name" )
  private String                  virtualName;

  @ManyToOne
  @JoinColumn( name = "metadata_image_dev_map_fk", insertable = false, updatable = false )
  private ImageInfo               parent;
  
  protected DeviceMapping( ) {
  }
  
  public DeviceMapping( ImageInfo parent, DeviceMappingType deviceMappingType, String deviceName, String virtualName ) {
    this.parent = parent;
    this.deviceName = deviceName;
    this.deviceMappingType = deviceMappingType;
    this.virtualName = virtualName;
  }
  
  public String getDeviceName( ) {
    return this.deviceName;
  }
  
  public void setDeviceName( String deviceName ) {
    this.deviceName = deviceName;
  }
  
  public ImageInfo getParent( ) {
    return this.parent;
  }
  
  public void setParent( ImageInfo parent ) {
    this.parent = parent;
  }

  public ImageMetadata.DeviceMappingType getDeviceMappingType( ) {
    return this.deviceMappingType;
  }

  protected void setDeviceMappingType( ImageMetadata.DeviceMappingType deviceMappingType ) {
    this.deviceMappingType = deviceMappingType;
  }

  public String getVirtualName( ) {
    return this.virtualName;
  }

  public void setVirtualName( String virtualName ) {
    this.virtualName = virtualName;
  }
}
