/*************************************************************************
 * Copyright 2009-2014 Ent. Services Development Corporation LP
 *
 * Redistribution and use of this software in source and binary forms,
 * with or without modification, are permitted provided that the
 * following conditions are met:
 *
 *   Redistributions of source code must retain the above copyright
 *   notice, this list of conditions and the following disclaimer.
 *
 *   Redistributions in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimer
 *   in the documentation and/or other materials provided with the
 *   distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 * ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 ************************************************************************/
package com.eucalyptus.imaging.common;

import java.util.List;
import java.util.ArrayList;
import java.net.URI;
import java.net.URISyntaxException;

import com.eucalyptus.component.ServiceConfiguration;
import com.eucalyptus.component.ServiceConfigurations;
import com.eucalyptus.objectstorage.ObjectStorage;
import com.eucalyptus.util.dns.DomainNames;

import org.xbill.DNS.Name;

public class UrlValidator {
  private final List<URI> osgUrls = new ArrayList<URI>();

  public UrlValidator() {
    List<ServiceConfiguration> osgs = ServiceConfigurations
        .list(ObjectStorage.class);
    for (ServiceConfiguration conf : osgs) {
      osgUrls.add(conf.getUri());
    }
  }

  /**
   * Checks if URL is pointing to a Eucalyptus
   * 
   * @param url
   * @return
   */
  public boolean isEucalyptusUrl(String url) {
    URI in = null;
    if (url == null)
      return false;

    try {
      in = new URI(url.startsWith("http") ? url : "http://" + url);
    } catch (URISyntaxException e) {
      return false;
    }

    if (DomainNames.isSystemSubdomain(Name.fromConstantString(in.getHost()
        + ".")))
      return true;

    for (URI u : osgUrls) {
      if (u.getHost().equalsIgnoreCase(in.getHost()))
        return true;
    }
    return false;
  }
}