/*************************************************************************
 * Copyright 2008 Regents of the University of California
 * Copyright 2009-2012 Ent. Services Development Corporation LP
 *
 * Redistribution and use of this software in source and binary forms,
 * with or without modification, are permitted provided that the
 * following conditions are met:
 *
 *   Redistributions of source code must retain the above copyright
 *   notice, this list of conditions and the following disclaimer.
 *
 *   Redistributions in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimer
 *   in the documentation and/or other materials provided with the
 *   distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 * ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE. USERS OF THIS SOFTWARE ACKNOWLEDGE
 * THE POSSIBLE PRESENCE OF OTHER OPEN SOURCE LICENSED MATERIAL,
 * COPYRIGHTED MATERIAL OR PATENTED MATERIAL IN THIS SOFTWARE,
 * AND IF ANY SUCH MATERIAL IS DISCOVERED THE PARTY DISCOVERING
 * IT MAY INFORM DR. RICH WOLSKI AT THE UNIVERSITY OF CALIFORNIA,
 * SANTA BARBARA WHO WILL THEN ASCERTAIN THE MOST APPROPRIATE REMEDY,
 * WHICH IN THE REGENTS' DISCRETION MAY INCLUDE, WITHOUT LIMITATION,
 * REPLACEMENT OF THE CODE SO IDENTIFIED, LICENSING OF THE CODE SO
 * IDENTIFIED, OR WITHDRAWAL OF THE CODE CAPABILITY TO THE EXTENT
 * NEEDED TO COMPLY WITH ANY SUCH LICENSES OR RIGHTS.
 ************************************************************************/

package com.eucalyptus.storage.common;

import java.io.ByteArrayOutputStream;
import java.io.RandomAccessFile;
import java.util.zip.GZIPOutputStream;

import org.apache.log4j.Logger;
import org.jboss.netty.buffer.ChannelBuffers;
import org.jboss.netty.handler.stream.ChunkedInput;

import edu.ucsb.eucalyptus.constants.IsData;

public class CompressedChunkedFile implements ChunkedInput, IsData {
  private Logger LOG = Logger.getLogger(CompressedChunkedFile.class);
  private RandomAccessFile file;
  private long offset;
  private int CHUNK_SIZE = 8192;
  private long fileLength;

  public CompressedChunkedFile(RandomAccessFile file, long fileLength) {
    this.file = file;
    this.offset = 0;
    this.fileLength = fileLength;
  }

  public CompressedChunkedFile(RandomAccessFile file, long start, long end, int chunkSize) {
    this.file = file;
    this.offset = start;
    this.fileLength = end;
    CHUNK_SIZE = chunkSize;
  }

  @Override
  public void close() throws Exception {
    file.close();
  }

  @Override
  public boolean hasNextChunk() throws Exception {
    return offset < fileLength && file.getChannel().isOpen();
  }

  @Override
  public Object nextChunk() throws Exception {
    long offset = this.offset;
    if (offset >= fileLength) {
      return null;
    }
    int chunkSize = (int) Math.min(CHUNK_SIZE, fileLength - offset);
    byte[] chunk = new byte[chunkSize];
    file.readFully(chunk);
    this.offset = offset + chunkSize;
    ByteArrayOutputStream out = new ByteArrayOutputStream();
    GZIPOutputStream zip = new GZIPOutputStream(out);
    zip.write(chunk);
    zip.close();
    return ChannelBuffers.wrappedBuffer(out.toByteArray());
  }

  @Override
  public boolean isEndOfInput() throws Exception {
    return !hasNextChunk();
  }
}
